/*******************************************************************************
 * Copyright (c) 2015, 2016 IRT AESE (IRT Saint Exupery).
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     Bassem Ouni and Pierre Gaufillet (IRT Saint Exupéry) - initial API and implementation
 *     
 *******************************************************************************/
package com.irtsaintexupery.capella2aadl.ui.popupMenus;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.ArrayList;
import java.util.List;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.resources.IContainer;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Status;
import org.eclipse.emf.common.util.URI;
import org.eclipse.jface.operation.IRunnableWithProgress;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.handlers.HandlerUtil;

import com.irtsaintexupery.capella2aadl.ui.Activator;
import com.irtsaintexupery.capella2aadl.ui.common.GenerateAll;

public class Capella2AADLTransformer extends AbstractHandler {

	public Object execute(ExecutionEvent event) throws ExecutionException {
		ISelection selection = HandlerUtil.getActiveWorkbenchWindow(event).getActivePage().getSelection();

		if (selection != null & selection instanceof IStructuredSelection) {

			IStructuredSelection strucSelection = (IStructuredSelection) selection;
			for (Object element : strucSelection.toList()) {
				if (element instanceof IFile) {
					final IFile input_file = (IFile) element;
					IRunnableWithProgress operation = new IRunnableWithProgress() {
						public void run(IProgressMonitor monitor) {
							try {
								try {
									IContainer target = input_file.getProject().getFolder("aadl-gen");

									if (target.exists()) {
										// Clean up target folder of its aadl
										// files

										for (IResource f : target.members()) {
											String ext = f.getFileExtension();
											if (ext != null ? ext.equals("aadl") : false)
												f.delete(true, null);
										}
									}

								} finally {
									input_file.getProject().refreshLocal(IResource.DEPTH_INFINITE, null);
								}
							} catch (CoreException e) {
								IStatus status = new Status(IStatus.ERROR, Activator.PLUGIN_ID, e.getMessage(), e);
								Activator.getDefault().getLog().log(status);
							}

							try {
								URI modelURI = URI.createPlatformResourceURI(input_file.getFullPath().toString(), true);
								IContainer target = input_file.getProject().getFolder("aadl-gen");

								GenerateAll generator = new GenerateAll(modelURI, target, getArguments());
								generator.doGenerate(monitor);
							} catch (IOException e) {
								IStatus status = new Status(IStatus.ERROR, Activator.PLUGIN_ID, e.getMessage(), e);
								Activator.getDefault().getLog().log(status);
							} finally {
								try {
									input_file.getProject().refreshLocal(IResource.DEPTH_INFINITE, monitor);
								} catch (CoreException e) {
									IStatus status = new Status(IStatus.ERROR, Activator.PLUGIN_ID, e.getMessage(), e);
									Activator.getDefault().getLog().log(status);
								}
							}
						}
					};
					try {
						PlatformUI.getWorkbench().getProgressService().run(true, true, operation);
					} catch (InvocationTargetException e) {
						IStatus status = new Status(IStatus.ERROR, Activator.PLUGIN_ID, e.getMessage(), e);
						Activator.getDefault().getLog().log(status);
					} catch (InterruptedException e) {
						IStatus status = new Status(IStatus.ERROR, Activator.PLUGIN_ID, e.getMessage(), e);
						Activator.getDefault().getLog().log(status);
					}

				}
			}
		}
		return null;
	}

	/**
	 * Computes the arguments of the generator.
	 * 
	 * @return the arguments
	 * @generated
	 */
	protected List<? extends Object> getArguments() {
		return new ArrayList<String>();
	}

}
